package cn.palmte.work.service;


import cn.palmte.work.config.activiti.ActConstant;
import cn.palmte.work.utils.ActUtil;
import com.alibaba.fastjson.JSONObject;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.delegate.DelegateExecution;
import org.activiti.engine.delegate.DelegateTask;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.runtime.ProcessInstanceQuery;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;


import java.util.Iterator;
import java.util.List;
import java.util.Set;


/**
 * 流程节点监听
 */
@Service
public class ActListenerService {
    private static final Logger logger = LoggerFactory.getLogger(ActListenerService.class);


    @Autowired
    private ActTaskDefService actTaskDefService;

    @Autowired
    private ProjectInstanceService projectInstanceService;

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private ActUtil actUtil;

    /**
     * 任务节点创建后监听 动态设置审批人
     *
     * @param delegateTask
     * @throws Exception
     */
    public void create(DelegateTask delegateTask) throws Exception {
        logger.info("--节点创建后监听-- name:{}, {}", delegateTask.getName(), JSONObject.toJSONString(delegateTask));

        String procDefId = delegateTask.getProcessDefinitionId();
        String procInsId = delegateTask.getProcessInstanceId();
        String taskDefKey = delegateTask.getTaskDefinitionKey();
        List<String> candidateUsers = actTaskDefService.findCandidateUsers(procDefId, procInsId, taskDefKey);
        logger.info("addCandidateUsers : {}", candidateUsers);

        ProcessInstanceQuery processInstanceQuery = runtimeService.createProcessInstanceQuery();
        List<ProcessInstance> list = processInstanceQuery.list();
        System.out.println(list);
        ProcessInstance processInstance = processInstanceQuery.processInstanceId(delegateTask.getProcessInstanceId()).singleResult();
        if (actUtil.isProjectProcessIns(processInstance)) {
            updateProjectApprover(candidateUsers, processInstance);
        }

        delegateTask.addCandidateUsers(candidateUsers);
    }


    /**
     * 多实例任务查询办理人
     *
     * @param delegateExecution
     * @throws Exception
     */
    public List<String> getMultiCandidateUsers(DelegateExecution delegateExecution) throws Exception {
        logger.info("--多实例任务查询办理人-- eventName:{}, {}", delegateExecution.getEventName(), delegateExecution);
        String procInsId = delegateExecution.getProcessInstanceId();
        String procDefId = delegateExecution.getProcessDefinitionId();
        String taskDefKey = delegateExecution.getCurrentActivityId();

        List<String> candidateUsers = actTaskDefService.findCandidateUsers(procDefId, procInsId, taskDefKey);
        logger.info("--getMultiCandidateUsers-- {}", candidateUsers);
        return candidateUsers;
    }


    /**
     * 更新项目审批人 四算业务需要
     *
     * @param candidateUsers
     * @param processInstance
     */
    private void updateProjectApprover(List<String> candidateUsers, ProcessInstance processInstance) {
        String businessKey = processInstance.getBusinessKey();
        logger.info("businessKey : {}", businessKey);

        if (candidateUsers != null && !candidateUsers.isEmpty() && businessKey != null) {
            try {
                int adminId = Integer.parseInt(candidateUsers.get(0));
                projectInstanceService.updateApprover(Integer.parseInt(businessKey), adminId);
            } catch (Exception e) {
                logger.error("", e);
            }
        }
    }
}
